#!/usr/bin/perl -w
#
#
# OpenPHPNuke: Great Web Portal System
# Professionell-Module
#
# Copyright (c) 2001-2006 by
# Heinz Hombergs heinz@hhombergs.de
# Stefan Kaletta stefan@kaletta.de
# Alexander Weber xweber@kamelfreunde.de
#
# This program is NOT free software.
# See LICENSE for details.
#
#

sub doSQL {
    
    my ($sql) = @_;
    
    my $qr = undef;
    
    push_el(\@main::el, 'doSQL()', 'Starting...');
    
    if (!defined($sql) || ($sql eq '')) {
        
        push_el(\@main::el, 'doSQL()', 'ERROR: Undefined SQL query !');
        
        return (-1, '');
        
    }
    
    if (!defined($main::db) || !ref($main::db)) {
        
        $main::db = DBI -> connect(@main::db_connect, {PrintError => 0});
        
        if ( !defined($main::db) ) {
            
            push_el(
                    \@main::el, 
                    'doSQL()', 
                    'ERROR: Unable to connect SQL server !'
                   ); 
            
            return (-1, '');
            
        }
    }
    
    if ($sql =~ /select/i) {
        
        $qr = $main::db -> selectall_arrayref($sql);
        
    } elsif ($sql =~ /show/i) {
        
        $qr = $main::db -> selectall_arrayref($sql);
        
    } else {
        
        $qr = $main::db -> do($sql);
        
    }
    
    if (defined($qr)) {
        
        push_el(\@main::el, 'doSQL()', 'Ending...');
        
        return (0, $qr);
        
    } else {
        
        push_el(\@main::el, 'doSQL()', 'ERROR: Incrrect SQL Query -> '.$main::db -> errstr);
        
        return (-1, '');
        
    }
    
}


sub serialize {
	my ($value) = @_;
	return serialize_value($value);
}

sub serialize_key {
	my ($value) = @_;
	my $s;
	
	if ($value =~ /^\d+$/) {
		if (abs($value) > ((2**32)/2-1)) {
			$s = "d:$value;";
		} else {
			$s = "i:$value;";
		}
	} else {
		my $vlen = length($value);
		$s = "s:$vlen:\"$value\";";
	}
	
	return $s;
}

sub serialize_value {
	my ($value) = @_;
	my $s;
	
	$value = defined($value) ? $value : '';
	
	if ( ref($value) =~ /hash/i) {
		my $num = keys(%{$value});
		$s .= "a:$num:{";
		foreach my $k ( keys(%$value) ) {
			$s .= serialize_key( $k );
			$s .= serialize_value( $$value{$k} );
		}
		$s .= "}";
	} elsif ( ref($value) =~ /array/i) {
		my $num = @{$value};
		$s .= "a:$num:{";
		for (my $k=0; $k < @$value; $k++ ) {
			$s .= serialize_key( $k );
			$s .= serialize_value( $$value[$k] );
		}
		$s .= "}";
	} elsif ($value =~ /^\-?(\d+)\.(\d+)$/) {
		$s = "d:$value;";
	} elsif ($value =~ /^\-?\d+$/) {
		if (abs($value) > ((2**32)/2-1)) {
			$s = "d:$value;";
		}
		else {
			$s = "i:$value;";
		}
	} elsif ($value eq "\0")  {
		$s = "N;";
	} else {
		my $vlen = length($value);
		$s = "s:$vlen:\"$value\";";
	}
	
	return $s;
}

sub unserialize {
	my ($string) = @_;
	return unserialize_value($string);
}

sub unserialize_value {
	my ($value) = @_;
	
	if ($value =~ /^a:(\d+):\{(.*)\}$/) {
		push_el(\@main::el, 'serialize()',"Unserializing array");
		
		my @chars = split(//, $2);
		
		push(@chars, ';');
		
		return unserialize_sub({}, $1*2, \@chars);
	} elsif ($value =~ /^s:(\d+):(.*);$/) {
		push_el(\@main::el, 'serialize()',"Unserializing single string ($value)");
		#$string =~ /^s:(\d+):/;
		return $2;
		#return substr($string, length($1) + 4, $1);
	} elsif ($value =~ /^(i|d):(\-?\d+\.?\d+?);$/) {
		push_el(\@main::el, 'serialize()',"Unserializing integer or double ($value)");
		return $2
		#substr($string, 2) + 0;
	} elsif ($value == /^N;$/i) {
		push_el(\@main::el, 'serialize()',"Unserializing NULL value ($value)");
		return "\0";
	} elsif ($value =~/^b:(\d+);$/) {
		push_el(\@main::el, 'serialize()',"Unserializing boolean value ($value)");
		return $1;
	} else {
		push_el(\@main::el, 'serialize()',"Unserializing BAD DATA!\n($value)");
		die("Trying to unserialize bad data!");
		return '';
	}

}

sub unserialize_sub {
	my ($hashref, $keys, $chars) = @_;
	my ($temp, $keyname, $skip, $strlen);
	my $mode = 'normal';		#default mode
	
	while ( defined(my $c = shift @{$chars}) ) {

		push_el(\@main::el, 'serialize()',"while $mode = $c");

		if ($mode eq 'string') {
			$skip = 1;

			push_el(\@main::el, 'serialize()'," ( skip = $skip ) ");

			if ($c =~ /\d+/) {
				$strlen = $strlen . $c;
			}
	
			if (($strlen =~ /\d+/) && ($c eq ':')) {
				push_el(\@main::el, 'serialize()',"[string] length = $strlen");
				$mode = 'readstring';
			}
	
		} elsif ($mode eq 'readstring') {
			next			if ($skip && ($skip-- > 0));
			$mode = 'set', next	if (!$strlen--);
	
			$temp .= $c;
	
		} elsif ($mode eq 'integer') {
			next 			if ($c eq ':');
			$mode = 'set', next	if ($c eq ';');
	
			if ($c =~ /\-|\d+/) {
				if ($c eq '-') {
					$temp .= $c unless $temp;
				} else {
					$temp .= $c;
				}
			}
		} elsif ($mode eq 'double') {
			next 			if ($c eq ':');
			$mode = 'set', next	if ($c eq ';');
	
			if ($c =~ /\-|\d+|\./) {
				if ($c eq '-') {
					$temp .= $c unless $temp;
				} else {
					$temp .= $c;
				}
			}
		} elsif ($mode eq 'null') {
	
			$temp = "\0";
			$mode = 'set', next;

		} elsif ($mode eq 'array') {
	
			if ($c eq '{') {
	
				$temp = unserialize_sub( $$hashref{$keyname}, ($temp*2), $chars );

				if(!defined($temp) || $temp eq "") {
					$temp = {};
				}
				
				$mode = 'set', next;
			} elsif ($c =~ /\d+/) {
				$temp = $temp . $c;
				push_el(\@main::el, 'serialize()',"array_length = $temp ($c)");
			}
		} elsif ($mode eq 'set') {
	
			if (defined($keyname)) {
				push_el(\@main::el, 'serialize()',"set [$keyname]=$temp");
	
				$$hashref{$keyname} = $temp;	
				
				undef $keyname;
			} else {
				push_el(\@main::el, 'serialize()',"set KEY=$temp");
				$keyname = $temp;
			}
	
			undef $temp;
			$mode = 'normal';
		}
	
		if ($mode eq 'normal') {
			$strlen = $temp = '';
	
			if (!$keys) {
				push_el(\@main::el, 'serialize()',"return normally, finished processing keys");
				return $hashref;
			}
	
			if ($c eq 'i') {
				$mode = 'integer';
				$keys--;
			}
			if ($c eq 'b') {
				$mode = 'integer';
				$keys--;
			}
			if ($c eq 'd') {
				$mode = 'double';
				$keys--;
			}
			if ($c eq 's') {
				$mode = 'string';
				$keys--;
			}
			if ($c eq 'a') {
				$mode = 'array';
				$keys--;
			}
			if ($c eq 'N') {
				$mode = 'null';
				$keys--;
			}
		}

	} 

	push_el(\@main::el, 'serialize()',"> unserialize_sub ran out of chars when it was expecting more.");
	die("unserialize_sub() ran out of characters when it was expecting more.");
	
	return 0;
}

sub dump_hash {
	my ($hashref, $offset) = @_;
	push_el(\@main::el, 'serialize()',"> dump_hash");
	
	foreach my $k (keys %{$hashref}) {
		print join ("",@$offset) . "$k = $$hashref{$k}\n";
		if (ref($$hashref{$k}) =~ /hash/i) {
			push (@$offset, "\t");
			&dump_hash($$hashref{$k}, $offset);
			pop @$offset;
		}
	}
	return 1;
}



return 1;