<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_DRIVERMSSQL_INCLUDED') ) {
	define ('_OPN_CLASS_DRIVERMSSQL_INCLUDED', 1);

	/**
	* Extends the SQL Connection class for MySQL Database access
	*/

	class SQLLayer_mssql extends SQLConnection {
		var $databaseType = 'mssql';
		var $dataProvider = 'mssql';
		var $replaceQuote = "''";
		// string to use to replace quotes
		var $metaDatabasesSQL = 'SELECT name FROM sysdatabases WHERE name <> \'master\'';
		var $metaTablesSQL = "SELECT name,case when type='U' then 'T' else 'V' end FROM sysobjects WHERE (type='U' or type='V') AND (name not in ('sysallocations','syscolumns','syscomments','sysdepends','sysfilegroups','sysfiles','sysfiles1','sysforeignkeys','sysfulltextcatalogs','sysindexes','sysindexkeys','sysmembers','sysobjects','syspermissions','sysprotects','sysreferences','systypes','sysusers','sysalternates','sysconstraints','syssegments','REFERENTIAL_CONSTRAINTS','CHECK_CONSTRAINTS','CONSTRAINT_TABLE_USAGE','CONSTRAINT_COLUMN_USAGE','VIEWS','VIEW_TABLE_USAGE','VIEW_COLUMN_USAGE','SCHEMATA','TABLES','TABLE_CONSTRAINTS','TABLE_PRIVILEGES','COLUMNS','COLUMN_DOMAIN_USAGE','COLUMN_PRIVILEGES','DOMAINS','DOMAIN_CONSTRAINTS','KEY_COLUMN_USAGE','dtproperties'))";
		var $metaColumnsSQL = 'SELECT c.name,t.name,c.length, (case when c.xusertype=61 then 0 else c.xprec end), (case when c.xusertype=61 then 0 else c.xscale end) from syscolumns c join systypes t on t.xusertype=c.xusertype join sysobjects o on o.id=c.id where o.name=\'%s\'';
		var $hasTop = 'top';
		// support mssql SELECT TOP 10 * FROM TABLE
		var $_has_mssql_init;
		var $maxParameterLen = 4000;
		var $_bindInputArray = true;

		/**
		* Constructor for SQLLayer_mssql
		*
		* @access public
		* @return void
		*/

		function SQLLayer_mssql () {
			if (!extension_loaded ('mssql') ) {
				trigger_error ('You must have the mssql extension installed.', E_USER_ERROR);
			}
			$this->_has_mssql_init = (strnatcmp (PHP_VERSION, '4.1.0') >= 0);
			$this->fetchMode = true;

		}

		/**
		*
		* @access public
		* @param string	$field
		* @param string	$ifNull
		* @return string
		*/

		function IfNull ($field, $ifNull) {
			return ' ISNULL(' . $field . ', ' . $ifNull . ') ';

		}

		/**
		*
		* @access private
		* @return mixed
		*/

		function _affectedrows () {
			return $this->GetOne ('SELECT @@rowcount');

		}

		/**
		*
		* @access public
		* @param string	$sql
		* @param int		$nrows
		* @param int		$offset
		* @param array		$inputarr
		* @return void
		*/

		function &SelectLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false) {
			if ($nrows>0 && $offset<=0) {
				$sql = preg_replace ('/(^\s*select\s+(distinctrow|distinct)?)/i', '\\1 ' . $this->hasTop . ' ' . $nrows . ' ', $sql);
				$rs = &$this->Execute ($sql, $inputarr);
			} else {
				$rs = &$this->_SelectLimit ($sql, $nrows, $offset, $inputarr);
			}
			return $rs;

		}

		/**
		* Start a transaction.
		*
		* @access public
		* @return boolean
		*/

		function BeginTrans () {
			if (!$this->transOff) {
				$this->transCnt += 1;
				$this->Execute ('BEGIN TRAN');
			}
			return true;

		}

		/**
		* Commit a transaction.
		*
		* @access public
		* @param bool	$ok
		* @return boolean
		*/

		function CommitTrans ($ok = true) {
			if (!$this->transOff) {
				if (!$ok) {
					return $this->RollbackTrans ();
				}
				if ($this->transCnt) {
					$this->transCnt -= 1;
				}
				$this->Execute ('COMMIT TRAN');
			}
			return true;

		}

		/**
		* Rollback a transaction.
		*
		* @access public
		* @return boolean
		*/

		function RollbackTrans () {
			if (!$this->transOff) {
				if ($this->transCnt) {
					$this->transCnt -= 1;
				}
				$this->Execute ('ROLLBACK TRAN');
			}
			return true;

		}

		/**
		* Locks a record.
		*
		* Usage:
		* $this->BeginTrans();
		* $this->RowLock('table1,table2','table1.id=33 and table2.id=table1.id'); # lock row 33 for both tables
		* 	some operation on both tables table1 and table2
		* $this->CommitTrans();
		*
		* @access public
		* @param string	$tables
		* @param string	$where
		* @return void
		*/

		function RowLock ($tables, $where) {
			if (!$this->transCnt) {
				$this->BeginTrans ();
			}
			return $this->GetOne ('SELECT top 1 null as ignore FROM ' . $tables . ' with (ROWLOCK,HOLDLOCK) WHERE ' . $where);

		}

		/**
		*
		* @access public
		* @param string	$table
		* @param bool		$primary
		* @return mixed boolean or array
		*/

		function &MetaIndexes ($table, $primary = false) {

			$table = $this->qstr ($table);
			$sql = 'SELECT i.name AS ind_name, C.name AS col_name, USER_NAME(O.uid) AS Owner, c.colid, k.Keyno, ' . 'CASE WHEN I.indid BETWEEN 1 AND 254 AND (I.status & 2048 = 2048 OR I.Status = 16402 AND O.XType = \'V\') THEN 1 ELSE 0 END AS IsPK, ' . 'CASE WHEN I.status & 2 = 2 THEN 1 ELSE 0 END AS IsUnique ' . 'FROM dbo.sysobjects o INNER JOIN dbo.sysindexes I ON o.id = i.id ' . 'INNER JOIN dbo.sysindexkeys K ON I.id = K.id AND I.Indid = K.Indid ' . 'INNER JOIN dbo.syscolumns c ON K.id = C.id AND K.colid = C.Colid ' . 'WHERE LEFT(i.name, 8) <> \'_WA_Sys_\' AND o.status >= 0 AND O.Name LIKE ' . $table . ' ' . 'ORDER BY O.name, I.Name, K.keyno';
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$rs = $this->Execute ($sql);
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if (!is_object ($rs) ) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$indexes = array ();
			while ($row = $rs->FetchRow () ) {
				if (!$primary && $row[5]) {
					continue;
				}
				$indexes[$row[0]]['unique'] = $row[6];
				$indexes[$row[0]]['columns'][] = $row[1];
			}
			unset ($row);
			return $indexes;

		}

		/**
		*
		* @access public
		* @param string	$table
		* @param string	$owner
		* @param bool		$upper
		* @return mixed boolean or array
		*/

		function MetaForeignKeys ($table, $owner = false, $upper = false) {

			$t = $owner;
			$table = $this->qstr (strtoupper ($table) );
			$sql = 'SELECT object_name(constid) AS constraint_name, col_name(fkeyid, fkey) AS column_name, ' . 'object_name(rkeyid) AS referenced_table_name, ' . 'col_name(rkeyid, rkey) AS referenced_column_name ' . 'FROM sysforeignkeys ' . 'WHERE upper(object_name(fkeyid)) = ' . $table . ' ' . 'ORDER BY constraint_name, referenced_table_name, keyno';
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$constraints = &$this->GetArray ($sql);
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			$arr = false;
			foreach ($constraints as $constr) {
				$arr[$constr[0]][$constr[2]][] = $constr[1] . '=' . $constr[3];
			}
			if (!$arr) {
				return false;
			}
			unset ($constraints);
			$arr2 = false;
			foreach ($arr as $k => $v) {
				foreach ($v as $a => $b) {
					if ($upper) {
						$a = strtoupper ($a);
					}
					$arr2[$a] = $b;
				}
			}
			unset ($arr);
			return $arr2;

		}

		/**
		*
		* @access public
		* @return mixed array or boolean
		*/

		function MetaDatabases () {
			if (@mssql_select_db ('master') ) {
				$qry = $this->metaDatabasesSQL;
				if ($rs = @mssql_query ($qry) ) {
					$tmpAr = array ();
					$ar = array ();
					while ($tmpAr = @mssql_fetch_row ($rs) ) {
						$ar[] = $tmpAr[0];
					}
					if (sizeof ($ar) ) {
						return $ar;
					}
				}
			}
			return false;

		}

		/**
		*
		* @access public
		* @param string	$table
		* @return boolean
		*/

		function &MetaPrimaryKeys ($table) {

			$schema = '';
			$this->_findschema ($table, $schema);
			if (!$schema) {
				$schema = $this->database;
			}
			if ($schema) {
				$schema = 'AND k.table_catalog LIKE \'' . $schema . '%\'';
			}
			$sql = 'SELECT distinct k.column_name,ordinal_position FROM information_schema.key_column_usage k, ' . 'information_schema.table_constraints tc ' . 'WHERE tc.constraint_name = k.constraint_name AND  tc.constraint_type = ' . '\'PRIMARY KEY\' AND k.table_name = \'' . $table . '\' ' . $schema . ' ORDER BY ordinal_position ';
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$a = $this->GetCol ($sql);
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}

			# !empty an der Stelle?
			if ($a && sizeof ($a)>0) {
				return $a;
			}

			/* return byRef required - so it must be a variable*/

			$false = false;
			return $false;

		}

		/**
		*
		* @access public
		* @param mixed	$ttype
		* @param bool	$showSchema
		* @param bool	$mask
		* @return void
		*/

		function &MetaTables ($ttype = false, $showSchema = false, $mask = false) {
			if ($mask) {
				$save = $this->metaTablesSQL;
				$mask = $this->qstr ( ($mask) );
				$this->metaTablesSQL .= ' AND name like ' . $mask;
			}
			$ret = &$this->_MetaTables ($ttype, $showSchema);
			if ($mask) {
				$this->metaTablesSQL = $save;
			}
			return $ret;

		}

		/**
		* Choose a database to connect to.
		*
		* @access public
		* @param string	$dbName
		* @return boolean
		*/

		function SelectDB ($dbName) {

			$success = false;
			if ($this->_connectionID) {
				$success = @mssql_select_db ($dbName);
				if ($success) {
					$this->databaseName = $dbName;
				}
			}
			return $success;

		}

		/**
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {
			if (empty ($this->_errorMsg) ) {
				$this->_errorMsg = mssql_get_last_message ();
			}
			return $this->_errorMsg;

		}

		/**
		*
		* @access public
		* @return mixed boolean or number
		*/

		function ErrorNo () {
			if (empty ($this->_errorMsg) ) {
				$this->_errorMsg = mssql_get_last_message ();
			}
			$id = @mssql_query ('SELECT @@ERROR', $this->_connectionID);
			if (!$id) {
				$this->_errorCode = false;
			}
			$arr = mssql_fetch_array ($id);
			@mssql_free_result ($id);
			if (is_array ($arr) ) {
				$this->_errorCode = $arr[0];
			} else {
				$this->_errorCode-1;
			}
			return $this->_errorCode;

		}

		/**
		* Connect to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return mixed unknown or boolean or number
		*/

		function _connect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$this->_connectionID = mssql_connect ($argHostname, $argUsername, $argPassword);
			if ($this->_connectionID === false) {
				return false;
			}
			if ($argDatabasename) {
				return $this->SelectDB ($argDatabasename);
			}
			return true;

		}

		/**
		* Persitent connection to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return mixed unknown or boolean or number
		*/

		function _pconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$this->_connectionID = mssql_pconnect ($argHostname, $argUsername, $argPassword);
			if ($this->_connectionID === false) {
				return false;
			}
			// persistent connections can forget to rollback on crash, so we do it here.
			if ($this->autoRollback) {
				$cnt = $this->GetOne ('SELECT @@TRANCOUNT');
				while (-- $cnt >= 0) {
					$this->Execute ('ROLLBACK TRAN');
				}
			}
			if ($argDatabasename) {
				return $this->SelectDB ($argDatabasename);
			}
			return true;

		}

		/**
		*
		* @access public
		* @param string	$sql
		* @return mixed unknown or array
		*/

		function Prepare ($sql) {

			$sqlarr = explode ('?', $sql);
			if (sizeof ($sqlarr)<=1) {
				return $sql;
			}
			$sql2 = $sqlarr[0];
			$max = sizeof ($sqlarr);
			for ($i = 1; $i< $max; $i++) {
				$sql2 .= '@P' . ($i-1) . $sqlarr[$i];
			}
			return array ($sql,
					$this->qstr ($sql2),
					$max);

		}

		/**
		*
		* @access public
		* @param string	$sql
		* @return mixed string or array
		*/

		function PrepareSP ($sql) {
			if (!$this->_has_mssql_init) {
				opnErrorHandler (E_WARNING, 'PrepareSP: mssql_init only available in PHP 4.1.0 and higher', __FILE__, __LINE__);
				return $sql;
			}
			$stmt = mssql_init ($sql, $this->_connectionID);
			if (!$stmt) {
				return $sql;
			}
			return array ($sql,
					$stmt);

		}

		/**
		*
		* Usage:
		* $stmt = $db->PrepareSP('SP_RUNSOMETHING'); -- takes 2 params, @myid and @group
		* # note that the parameter does not have @ in front!
		* $db->Parameter($stmt,$id,'myid');
		* $db->Parameter($stmt,$group,'group',false,64);
		* $db->Execute($stmt);
		*
		* @access public
		* @param array		$stmt		Statement returned by Prepare() or PrepareSP().
		* @param var		$var		PHP variable to bind to. Can set to null (for isNull support).
		* @param string	$name		Name of stored procedure variable name to bind to.
		* @param int		$isOutput	Indicates direction of parameter 0/false=IN  1=OUT  2= IN/OUT. This is ignored in oci8.
		* @param int		$maxLen		Holds an maximum length of the variable.
		* @param int		$type		The data type of $var. Legal values depend on driver.
		* @return mixed unknown or bool
		*/

		function Parameter (&$stmt, &$var, $name, $isOutput = false, $maxLen = 4000, $type = false) {
			if (!$this->_has_mssql_init) {
				opnErrorHandler (E_WARNING, 'Parameter: mssql_bind only available in PHP 4.1.0 or higher', __FILE__, __LINE__);
				return '';
			}
			$isNull = is_null ($var);
			if ($type === false)
			switch (gettype ($var) ) {
				default:
				case 'string':
					$type = SQLCHAR;
					break;
				case 'double':
					$type = SQLFLT8;
					break;
				case 'integer':
					$type = SQLINT4;
					break;
				case 'boolean':
					$type = SQLINT1;
					break;

				# SQLBIT not supported in 4.1.0
			}
			if ($name !== 'RETVAL') {
				$name = '@' . $name;
			}
			return mssql_bind ($stmt[1], $name, $var, $type, $isOutput, $isNull, $maxLen);

		}

		/**
		* Execute the query.
		*
		* @access private
		* @param string	$sql
		* @param array		$inputarr
		* @return mixed resource or mixed
		*/

		function _query ($sql, $inputarr) {

			$this->_errorMsg = false;
			if (is_array ($inputarr) ) {

				# bind input params with sp_executesql:

				# see http://www.quest-pipelines.com/newsletter-v3/0402_F.htm

				# works only with sql server 7 and newer
				if (!is_array ($sql) ) {
					$sql = $this->Prepare ($sql);
				}
				$params = '';
				$decl = '';
				$i = 0;
				foreach ($inputarr as $v) {
					if ($decl) {
						$decl .= ', ';
						$params .= ', ';
					}
					if (is_string ($v) ) {
						$len = strlen ($v);
						if ($len == 0) {
							$len = 1;
						}
						if ($len>4000) {
							// NVARCHAR is max 4000 chars. Let's use NTEXT
							$decl .= "@P " . $i . "NTEXT";
						} else {
							$decl .= "@P$i NVARCHAR($len)";
						}
						$params .= "@P$i=N" . (strncmp ($v, "'", 1) == 0? $v : $this->qstr ($v) );
					} elseif (is_integer ($v) ) {
						$decl .= "@P$i INT";
						$params .= "@P$i=" . $v;
					} elseif (is_float ($v) ) {
						$decl .= "@P$i FLOAT";
						$params .= "@P$i=" . $v;
					} elseif (is_bool ($v) ) {
						$decl .= "@P$i INT";

						# Used INT just in case BIT in not supported on the user's MSSQL version. It will cast appropriately.

						$params .= "@P$i=" . ( ($v)?'1' : '0');

						# True == 1 in MSSQL BIT fields and acceptable for storing logical true in an int field
					} else {
						$decl .= "@P$i CHAR";

						# Used char because a type is required even when the value is to be NULL.

						$params .= "@P$i=NULL";
					}
					$i += 1;
				}
				$decl = $this->qstr ($decl);
				$rez = mssql_query ('sp_executesql N' . $sql[1] . ',N' . $decl . ',' . $params);
			} elseif (is_array ($sql) ) {

				# PrepareSP()

				$rez = mssql_execute ($sql[1]);
			} else {
				$rez = mssql_query ($sql, $this->_connectionID);
			}
			return $rez;

		}

		/**
		* Close the connection.
		*
		* @access private
		* @return number
		*/

		function _close () {
			if ($this->transCnt) {
				$this->RollbackTrans ();
			}
			$rez = @mssql_close ($this->_connectionID);
			$this->_connectionID = false;
			return $rez;

		}

	}
	// class SQLLayer_mssql

	/**
	* Extends the RecordSet class that represents the dataset returned by the database
	* for MySQL Database access
	*
	* @var string	$databaseType
	* @var boolean	$canSeek
	*/

	class SQLLayerRecordSet_mssql extends SQLLayerRecordSet {
		var $databaseType = 'mssql';
		var $canSeek = true;

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$queryID
		* @param bool		$mode
		* @return void
		*/

		function SQLLayerRecordSet_mssql ($queryID, $mode = true) {
			if ($mode) {
				$this->fetchMode = MSSQL_ASSOC;
			} else {
				$this->fetchMode = MSSQL_NUM;
			}
			$this->SQLLayerRecordSet ($queryID);

		}

		/**
		* Init the recordset.
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = @mssql_num_rows ($this->_queryID);
			$this->_numOfFields = @mssql_num_fields ($this->_queryID);

		}

		/**
		* Get next resultset - requires PHP 4.0.5 or later
		*
		* @access public
		* @return boolean
		*/

		function NextRecordSet () {
			if (!mssql_next_result ($this->_queryID) ) {
				return false;
			}
			$this->_inited = false;
			$this->bind = false;
			$this->_currentRow = -1;
			$this->Init ();
			return true;

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return unknown
		*/

		function Fields ($colname) {
			if ($this->fetchMode != MSSQL_NUM) {
				return $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0; $i< $this->_numOfFields; $i++) {
					$o = $this->FetchField ($i);
					$this->bind[strtolower ($o->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return mixed unknown or number
		*/

		function FetchField ($fieldOffset = -1) {
			if ($fieldOffset != -1) {
				return @mssql_fetch_field ($this->_queryID, $fieldOffset);
			}
			if ($fieldOffset == -1) {

				/*	The $fieldOffset argument is not provided thus its -1 	*/
				return @mssql_fetch_field ($this->_queryID);
			}
			return null;

		}

		/**
		* Adjusts the result pointer to an arbitary row in the result.
		*
		* @access private
		* @param int	$row
		* @return number
		*/

		function _seek ($row) {
			return @mssql_data_seek ($this->_queryID, $row);

		}

		/**
		* Fetch a record.
		*
		* @access private
		* @param bool	$ignore_fields
		* @return mixed array or number
		*/

		function _fetch () {

			$this->fields = @mssql_fetch_array ($this->_queryID, $this->fetchMode);
			if ($this->fetchMode == MSSQL_ASSOC) {
				if (is_array ($this->fields) ) {
					$this->_convert_to_lowercase ();
				}
			}
			if (is_array ($this->fields) ) {
				foreach ($this->fields as $k => $v) {
					if (is_string ($v) ) {
						$this->fields[$k] = rtrim ($v);
					}
				}
			}
			return $this->fields;

		}

		/**
		* Close the recordset.
		*
		* @access private
		* @return bool
		*/

		function _close () {

			$rez = mssql_free_result ($this->_queryID);
			$this->_queryID = false;
			return $rez;

		}

	}
	// class SQLLayerRecordSet_mssql

	class opn_mssql extends OPN_SQL {
		var $DropTable = 'DROP TABLE %s';
		var $DropIndex = 'DROP INDEX %s';

		function opn_mssql () {

			$this->CanOptimize = false;
			$this->CanRepair = false;
			$this->HasStatus = false;
			$this->CanAlterColumnAdd = false;
			$this->CanAlterColumnChange = false;
			$this->IndexCreate = 'CREATE %s INDEX idx%sidx%s ON %s %s';
			$this->CanAddRepairField = false;
			$this->CanChangeRepairField = false;
			$this->CanExecuteColumnAdd = false;
			$this->CanExecuteColumnChange = false;

		}

		function TableDrop ($tablename) {
			return 'DROP TABLE ' . $tablename;

		}

		/*
		function TableComment($tablename, $comment) {
		return '';
		// Hier bitte ergnzen fehlt noch
		}

		function TableLock($tablename, $lockmode=_OPNSQL_TABLOCK_ACCESS_EXCLUSIVE) {
		global $opnConfig;
		if ( (isset($opnConfig['opn_db_lock_use'])) AND ($opnConfig['opn_db_lock_use']==1) ) {
		$opnConfig['database']->Execute('LOCK TABLES '.$tablename.' EXCLUSIVE');
		}
		}

		function IndexDrop($index,$table='') {
		return 'DROP INDEX '.$index;
		}

		function PrimaryIndexDrop($table) {
		return 'ALTER TABLE '.$table.' DROP PRIMARY KEY';
		}

		*/

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {
			return sprintf ($this->IndexCreate, $unique, $idxname, $idxcounter, $table, $fields);

		}

		function GetIndex ($table) {
			return "SELECT Index_Name, TableName FROM USER_INDEXES WHERE Table_Name='$table' AND Index_Name NOT LIKE 'PK\_%'";

		}

		/*
		function TableUnLock($tablename) {
		global $opnConfig;
		if ( (isset($opnConfig['opn_db_lock_use'])) AND ($opnConfig['opn_db_lock_use']==1) ) {
		$opnConfig['database']->Execute('COMMIT WORK');
		}
		}
		*/

		function TableRename ($oldname, $newname) {
			return 'IF Exists (Select * from dbo.sysobjects where id = object_id(N\'[dbo].[' . $oldname . ']\') and OBJECTPROPERTY(id, N\'IsUserTable\') = 1) Exec sp_rename \'' . $oldname . '\', \'' . $newname . '\'';

		}

		/*
		function GetPrimaryKey($fields,$table='') {
		$hlp='constraint PK_'.$table.' PRIMARY KEY (';
		$j=count($fields);
		for ($i=0; $i < $j; $i++) {
		$hlp.=$fields[$i].',';
		}
		return substr_replace($hlp,')',strlen($hlp));
		}
		*/

		function GetDBType ($datatype, $length = 0, $default = null, $null = false, $decimal = 0) {

			global $opnConfig;
			if ( (isset ($opnConfig['return_tableinfo_as_array']) ) && ($opnConfig['return_tableinfo_as_array'] === TRUE) ) {
				if ($decimal == 0) {
					$decimal = null;
				}
				return $opnConfig['opnSQL']->BuildField ('', $length, $datatype, ! $null, !is_null ($default), $default, $decimal);
			}
			$hlp = '';
			if ($length == '') {
				$length = 0;
			}
			switch ($datatype) {
				case _OPNSQL_TINYINT:
					$hlp .= 'TINYINT';
					break;
				case _OPNSQL_SMALLINT:
					$hlp .= 'SMALLINT';
					break;
				case _OPNSQL_MEDIUMINT:
				case _OPNSQL_INT:
					$hlp .= 'INT';
					break;
				case _OPNSQL_BIGINT:
					$hlp .= 'INTEGER (' . $length . ')';
					break;
				case _OPNSQL_FLOAT:
				case _OPNSQL_DOUBLE:
				case _OPNSQL_NUMERIC:
					if ($length != 0) {
						$hlp .= 'DECIMAL (' . $length . ',' . $decimal . ')';
					} else {
						$hlp .= 'DECIMAL';
					}
					break;
				case _OPNSQL_DATE:
				case _OPNSQL_DATETIME:
				case _OPNSQL_TIMESTAMP:
				case _OPNSQL_TIME:
					$hlp .= 'DATETIME';
					break;
				case _OPNSQL_CHAR:
					$hlp .= 'CHAR (' . $length . ')';
					break;
				case _OPNSQL_VARCHAR:
					$hlp .= 'VARCHAR (' . $length . ')';
					break;
				case _OPNSQL_TINYBLOB:
				case _OPNSQL_BLOB:
				case _OPNSQL_MEDIUMBLOB:
				case _OPNSQL_BIGBLOB:

					#				$hlp.='IMAGE';

					#				break;
				case _OPNSQL_TINYTEXT:
				case _OPNSQL_TEXT:
				case _OPNSQL_MEDIUMTEXT:
				case _OPNSQL_BIGTEXT:
					$hlp .= 'TEXT';
					break;
			}
			// case
			if ($default !== NULL) {
				$hlp .= ' DEFAULT ';
				if (strlen ($default) == 0) {
					$hlp .= "''";
				} else {
					$hlp .= "'" . $default . "'";
				}
			}
			if ($null) {
			} else {
				$hlp .= ' NOT NULL';
			}
			return $hlp;

		}

	}
}

?>