<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

if (!defined ('_OPN_MAINFILE_INCLUDED') ) { die(); }

include_once (_OPN_ROOT_PATH . 'modules/bug_tracking/include/graphviz_api.php');

class BugData {
	var $is_descendant = false;
	var $parents = array ();
	var $childs = array ();

	function SetDesendant ($bool) {

		$this->is_descendant = $bool;

	}

	function SetParents ($parents) {

		$this->parents = $parents;

	}

	function SetChilds ($childs) {

		$this->cchilds = $childs;

	}

}

function BugTracking_php_version_at_least ($p_version_string) {

	$t_curver = array_pad (explode ('.', phpversion () ), 3, 0);
	$t_minver = array_pad (explode ('.', $p_version_string), 3, 0);
	for ($i = 0; $i<3; $i = $i+1) {
		$t_cur = (int) $t_curver[$i];
		$t_min = (int) $t_minver[$i];
		if ($t_cur<$t_min) {
			return false;
		}
		if ($t_cur>$t_min) {
			return true;
		}
	}
	// if we get here, the versions must match exactly so:
	return true;

}

function BugTracking_relgraph_generate_dep_graph ($bug_id, $sel_project, $p_horizontal) {

	global $opnConfig;
	// List of visited issues and their data.
	$v_bug_list = array ();
	// Firstly, we visit all ascendant issues and all descendant issues
	// and collect all the necessary data in the $v_bug_list variable.
	// We do not visit other descendants of our parents, neither other
	// ascendants of our children, to avoid displaying too much unrelated
	// issues. We still collect the information about those relationships,
	// so, if these issues happen to be visited also, relationship links
	// will be preserved.
	// The first issue in the list is the one we are parting from.
	$v_bug_list[$bug_id] = & new BugData;
	$v_bug_list[$bug_id]->SetDesendant (true);
	// Now we visit all ascendants of the root issue.
	$t_relationships = BugTracking_relationship_get_all_dest ($bug_id);
	foreach ($t_relationships as $t_relationship) {
		if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
			continue;
		}
		$v_bug_list[$bug_id]->parents[] = $t_relationship->src_bug_id;
		BugTracking_relgraph_add_parent ($v_bug_list, $t_relationship->src_bug_id);
	}
	$t_relationships = BugTracking_relationship_get_all_src ($bug_id);
	foreach ($t_relationships as $t_relationship) {
		if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
			continue;
		}
		$v_bug_list[$bug_id]->childs[] = $t_relationship->dest_bug_id;
		BugTracking_relgraph_add_child ($v_bug_list, $t_relationship->dest_bug_id);
	}
	// We have already collected all the information we need to generate
	// the graph. Now it is the matter to create a Digraph object and
	// store the information there, along with graph formatting attributes.
	$t_id_string = $bug_id;
	$t_graph_fontname = $opnConfig['bugtracking_graph_font'];
	$t_graph_fontsize = $opnConfig['bugtracking_graph_fontsize'];
	$t_graph_fontpath = $opnConfig['bugtracking_graph_fontpath'];
	$t_view_on_click = $opnConfig['bugtracking_graph_view_on_click'];
	$t_dot_tool = $opnConfig['bugtracking_graph_dot_tool'];
	$t_graph_attributes = array ();
	if (!empty ($t_graph_fontpath) ) {
		$t_graph_attributes['fontpath'] = $t_graph_fontpath;
	}
	if ($p_horizontal) {
		$t_graph_attributes['rankdir'] = 'LR';
		$t_graph_orientation = 'horizontal';
	} else {
		$t_graph_orientation = 'vertical';
	}
	$t_graph = new Digraph ($t_id_string, $t_graph_attributes, $t_dot_tool);
	$t_graph->set_default_node_attr (array ('fontname' => $t_graph_fontname,
						'fontsize' => $t_graph_fontsize,
						'shape' => 'record',
						'style' => 'filled',
						'height' => '0.2',
						'width' => '0.4') );
	$t_graph->set_default_edge_attr (array ('style' => 'solid',
						'color' => '#C00000',
						'dir' => 'back') );
	$url[0] = $opnConfig['opn_url'] . '/modules/bug_tracking/index.php';
	$url['sel_project'] = $sel_project;
	// Add all issue nodes and edges to the graph.
	foreach ($v_bug_list as $t_related_bug_id => $t_related_bug) {
		$t_id_string = $t_related_bug_id;
		$url['bug_id'] = $t_related_bug_id;
		if ($t_view_on_click) {
			$url['op'] = 'view_bug';
		} else {
			$url['op'] = 'graph_dependency';
			$url['graphr'] = $t_graph_orientation;
		}
		$t_url = encodeurl ($url);
		BugTracking_relgraph_add_bug_to_graph ($t_graph, $t_id_string, $t_url, $t_related_bug_id == $bug_id);

		# Now add all relationship edges to the graph.

		foreach ($v_bug_list[$t_related_bug_id]->parents as $t_parent_id) {

			# Do not create edges for unvisited bugs.
			if (!isset ($v_bug_list[$t_parent_id]) ) {
				continue;
			}
			$t_parent_node = $t_parent_id;
			$t_graph->add_edge ($t_parent_node, $t_id_string);
		}
	}
	return $t_graph;

}

function BugTracking_relgraph_generate_rel_graph ($p_bug_id, $sel_project) {

	global $opnConfig;
	// List of visited issues and their data.
	$v_bug_list = array ();
	$v_rel_list = array ();
	// Queue for breadth-first
	$v_queue = array ();
	// Now we visit all related issues.
	$t_max_depth = $opnConfig['bugtracking_graph_max_depth'];
	// Put the first element into queue.
	array_push ($v_queue, array (0,
				$p_bug_id) );
	// And now we proccess it
	while (!empty ($v_queue) ) {
		list ($t_depth, $t_id) = array_shift ($v_queue);
		if (isset ($v_bug_list[$t_id]) ) {
			continue;
		}
		if (!BugTracking_bug_exists ($t_id) ) {
			continue;
		}
		$v_bug_list[$t_id] = $t_id;
		$t_relationships = BugTracking_relationship_get_all_src ($t_id);
		foreach ($t_relationships as $t_relationship) {
			$t_dst = $t_relationship->dest_bug_id;
			if (_OPN_BUG_RELATION_DEPENDANT_ON == $t_relationship->type) {
				$v_rel_list[$t_id][$t_dst] = _OPN_BUG_RELATION_DEPENDANT_ON;
				$v_rel_list[$t_dst][$t_id] = _OPN_BUG_RELATION_DEPENDANT_OF;
			} else {
				$v_rel_list[$t_id][$t_dst] = $t_relationship->type;
				$v_rel_list[$t_dst][$t_id] = $t_relationship->type;
			}
			if ($t_depth<$t_max_depth) {
				array_push ($v_queue, array ($t_depth+1,
							$t_dst) );
			}
		}
		$t_relationships = BugTracking_relationship_get_all_dest ($t_id);
		foreach ($t_relationships as $t_relationship) {
			$t_dst = $t_relationship->src_bug_id;
			if (_OPN_BUG_RELATION_DEPENDANT_OF == $t_relationship->type) {
				$v_rel_list[$t_id][$t_dst] = _OPN_BUG_RELATION_DEPENDANT_OF;
				$v_rel_list[$t_dst][$t_id] = _OPN_BUG_RELATION_DEPENDANT_ON;
			} else {
				$v_rel_list[$t_id][$t_dst] = $t_relationship->type;
				$v_rel_list[$t_dst][$t_id] = $t_relationship->type;
			}
			if ($t_depth<$t_max_depth) {
				array_push ($v_queue, array ($t_depth+1,
							$t_dst) );
			}
		}
	}
	$t_id_string = $p_bug_id;
	$t_graph_fontname = $opnConfig['bugtracking_graph_font'];
	$t_graph_fontsize = $opnConfig['bugtracking_graph_fontsize'];
	$t_graph_fontpath = $opnConfig['bugtracking_graph_fontpath'];
	$t_view_on_click = $opnConfig['bugtracking_graph_view_on_click'];
	$t_neato_tool = $opnConfig['bugtracking_graph_neato_tool'];
	$t_graph_attributes = array ();
	if (!empty ($t_graph_fontpath) ) {
		$t_graph_attributes['fontpath'] = $t_graph_fontpath;
	}
	$t_graph = new Graph ($t_id_string, $t_graph_attributes, $t_neato_tool);
	$t_graph->set_default_node_attr (array ('fontname' => $t_graph_fontname,
						'fontsize' => $t_graph_fontsize,
						'shape' => 'record',
						'style' => 'filled',
						'height' => '0.2',
						'width' => '0.4') );
	$t_graph->set_default_edge_attr (array ('style' => 'solid',
						'color' => '#0000C0',
						'dir' => 'none') );
	$url[0] = $opnConfig['opn_url'] . '/modules/bug_tracking/index.php';
	$url['sel_project'] = $sel_project;
	// Add all issue nodes and edges to the graph.
	foreach ($v_bug_list as $t_id => $t_bug) {
		$t_id_string = $t_id;
		$url['bug_id'] = $t_id;
		if ($t_view_on_click) {
			$url['op'] = 'view_bug';
		} else {
			$url['op'] = 'graph_relation';
		}
		$t_url = encodeurl ($url);
		BugTracking_relgraph_add_bug_to_graph ($t_graph, $t_id_string, $t_url, $t_id == $p_bug_id);
		// Now add all relationship edges to the graph.
		if (isset ($v_rel_list[$t_id]) ) {
			foreach ($v_rel_list[$t_id] as $t_dst => $t_relation) {
				// Do not create edges for unvisited bugs.
				if (!isset ($v_bug_list[$t_dst]) ) {
					continue;
				}
				// avoid double links
				if ($t_dst<$t_id) {
					continue;
				}
				$t_related_id = $t_dst;
				switch ($t_relation) {
					case _OPN_BUG_RELATION_DUPLICATE_OF:
					case _OPN_BUG_RELATION_HAS_DUPLICATE:
						$t_edge_style = array ('style' => 'dashed',
									'color' => '#808080');
						break;
					case _OPN_BUG_RELATION_DEPENDANT_OF:
						$t_edge_style = array ('color' => '#C00000',
									'dir' => 'back');
						break;
					case _OPN_BUG_RELATION_DEPENDANT_ON:
						$t_edge_style = array ('color' => '#C00000',
									'dir' => 'forward');
						break;
					default:
						$t_edge_style = array ();
						break;
				}
				$t_graph->add_edge ($t_id_string, $t_related_id, $t_edge_style);
			}
		}
	}
	return $t_graph;

}

function BugTracking_relationship_get_all_dest ($p_dest_bug_id) {

	global $opnConfig, $opnTables, $bugs;

	$query = 'SELECT br.relation_id AS relation_id, br.relation_type AS relation_type, br.source_bug_id AS source_bug_id' . ', br.destination_bug_id as destination_bug_id, b.project_id AS project_id FROM ' . $opnTables['bugs_relation'] . ' br ' . 'INNER JOIN ' . $opnTables['bugs'] . ' b ON br.source_bug_id = b.bug_id ' . 'WHERE br.destination_bug_id=' . $p_dest_bug_id . ' ORDER BY br.relation_type, br.relation_id';
	$result = $opnConfig['database']->Execute ($query);
	$bug = $bugs->RetrieveSingle ($p_dest_bug_id);
	$t_des_project_id = $bug['project_id'];
	$t_bug_relationship_data = array (new BugRelationshipData);
	$t_relationship_count = $result->RecordCount ();
	$i = 0;
	while (! $result->EOF) {
		$t_bug_relationship_data[$i]->SetId ($result->fields['relation_id']);
		$t_bug_relationship_data[$i]->SetSrcBug ($result->fields['source_bug_id']);
		$t_bug_relationship_data[$i]->SetSrcProject ($result->fields['project_id']);
		$t_bug_relationship_data[$i]->SetDestBug ($result->fields['destination_bug_id']);
		$t_bug_relationship_data[$i]->SetDestProject ($t_des_project_id);
		$t_bug_relationship_data[$i]->SetType ($result->fields['relation_type']);
		$i++;
		$result->MoveNext ();
	}
	unset ($t_bug_relationship_data[$t_relationship_count]);
	return $t_bug_relationship_data;

}

function BugTracking_relgraph_add_bug_to_graph (&$p_graph, $p_bug_id, $p_url = null, $p_highlight = false) {

	global $bugs, $bugvalues, $opnConfig;

	$t_node_attributes = array ();
	$t_node_attributes['label'] = $p_bug_id;
	if ($p_highlight) {
		$t_node_attributes['color'] = '#0000FF';
		$t_node_attributes['style'] = 'bold, filled';
	} else {
		$t_node_attributes['color'] = 'black';
		$t_node_attributes['style'] = 'filled';
	}
	$bug = $bugs->RetrieveSingle ($p_bug_id);
	switch ($bug['status']) {
		case _OPN_BUG_STATUS_NEW:
			$color = $opnConfig['bugtrackingnew'];
			break;
		case _OPN_BUG_STATUS_FEEDBACK:
			$color = $opnConfig['bugtrackingfeedback'];
			break;
		case _OPN_BUG_STATUS_ACKNOWLEDGED:
			$color = $opnConfig['bugtrackingacknowledged'];
			break;
		case _OPN_BUG_STATUS_CONFIRMED:
			$color = $opnConfig['bugtrackingconfirmed'];
			break;
		case _OPN_BUG_STATUS_ASSIGNED:
			$color = $opnConfig['bugtrackingassigned'];
			break;
		case _OPN_BUG_STATUS_RESOLVED:
			$color = $opnConfig['bugtrackingresolved'];
			break;
		case _OPN_BUG_STATUS_CLOSED:
			$color = $opnConfig['bugtrackingclosed'];
			break;
	}
	// switch
	$t_node_attributes['fillcolor'] = $color;
	if (null !== $p_url) {
		$t_node_attributes['URL'] = $p_url;
	}
	$t_summary = $bug['summary'];
	$t_status = $bugvalues['status'][$bug['status']];
	$t_node_attributes['tooltip'] = '[' . $t_status . '] ' . $t_summary;
	$p_graph->add_node ($p_bug_id, $t_node_attributes);

}

function BugTracking_relgraph_output_map ($p_graph, $p_name) {

	$p_graph->output ('cmapx', false, $p_name);
	// return $check;

}

function BugTracking_relgraph_add_child (&$p_bug_list, $p_bug_id) {
	// Check if the issue is already in the issue list.
	if (isset ($p_bug_list[$p_bug_id]) ) {
		// The issue is in the list, but we cannot discard it since it
		// may be a parent issue (whose children were not visited).
		if (!$p_bug_list[$p_bug_id]->is_descendant) {
			// Yes, we visited this issue as a parent... This is the case
			// where someone set up a cyclic relationship (I really hope
			// nobody ever do this, but should keep sanity) for this
			// issue. We just have to finish the job, visiting all issues
			// that were already listed by _add_parent().
			$p_bug_list[$p_bug_id]->SetDesendant (true);
			foreach ($p_bug_list[$p_bug_id]->childs as $t_child)BugTracking_relgraph_add_child ($p_bug_id, $t_child);
		}
	} else {
		if (!BugTracking_bug_exists ($p_bug_id) ) {
			return false;
		}
		// The issue is not in the list, proceed as usual.
		// Check if the issue really exists and we have access to it.
		// If not, it is like it didn't exist.
		// Add the issue to the list.
		$p_bug_list[$p_bug_id] = & new BugData;
		$p_bug_list[$p_bug_id]->SetDesendant (true);
		// Add all parent issues to the list of parents. Do not visit them
		// for the same reason we didn't visit the children of all
		// ancestors.
		$t_relationships = BugTracking_relationship_get_all_dest ($p_bug_id);
		foreach ($t_relationships as $t_relationship) {
			if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
				continue;
			}
			$p_bug_list[$p_bug_id]->parents[] = $t_relationship->src_bug_id;
		}
		// Add all child issues to the list of children and visit them
		// recursively.
		$t_relationships = BugTracking_relationship_get_all_src ($p_bug_id);
		foreach ($t_relationships as $t_relationship) {
			if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
				continue;
			}
			$p_bug_list[$p_bug_id]->childs[] = $t_relationship->dest_bug_id;
			BugTracking_relgraph_add_child ($p_bug_list, $t_relationship->dest_bug_id);
		}
	}
	return true;

}

function BugTracking_relgraph_add_parent (&$p_bug_list, $p_bug_id) {
	// If the issue is already in the list, we already visited it, just
	// leave.
	if (isset ($p_bug_list[$p_bug_id]) ) {
		return true;
	}
	if (!BugTracking_bug_exists ($p_bug_id) ) {
		return false;
	}
	// Add the issue to the list.
	$p_bug_list[$p_bug_id] = & new BugData;
	// Add all parent issues to the list of parents and visit them
	// recursively.
	$t_relationships = BugTracking_relationship_get_all_dest ($p_bug_id);
	foreach ($t_relationships as $t_relationship) {
		if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
			continue;
		}
		$p_bug_list[$p_bug_id]->parents[] = $t_relationship->src_bug_id;
		BugTracking_relgraph_add_parent ($p_bug_list, $t_relationship->src_bug_id);
	}
	// Add all child issues to the list of children. Do not visit them
	// since this will add too much data that is unrelated to the original
	// issue, and has a potential to generate really huge graphs.
	$t_relationships = BugTracking_relationship_get_all_src ($p_bug_id);
	foreach ($t_relationships as $t_relationship) {
		if ( ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_ON) && ($t_relationship->type != _OPN_BUG_RELATION_DEPENDANT_OF) ) {
			continue;
		}
		$p_bug_list[$p_bug_id]->childs[] = $t_relationship->dest_bug_id;
	}
	return true;

}

?>