<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

if (!defined ('_OPN_MAINFILE_INCLUDED') ) { die(); }

include_once (_OPN_ROOT_PATH . 'modules/geodb/class/class.map.php');
include_once (_OPN_ROOT_PATH . 'modules/geodb/class/class.e00.php');

/**
* geodb_map
*
* this class provides functions to generate images from
* GIS data and GeoObjects.
*
* @access   public
* @package  Geo
*/

class geodb_map extends e00 {
	var $colors = array ();
	var $latitudeMin;
	var $latitudeMax;
	var $longitudeMin;
	var $longitudeMax;
	var $objects = array ();
	var $imageMap = array ();
	var $color = array ();
	var $radius = 4;

	/**
	* constructor
	*
	* @param   mixed  $x  image-width (int) or path to image (string)
	* @param   int    $y  image-height
	* @return  void
	*/

	function geodb_map ($x = false, $y = false) {

		$this->e00 ($x, $y);
		$this->MapSetColor ('white', $this->allocateColor (255, 255, 255) );
		$this->MapSetColor ('red', $this->allocateColor (255, 0, 0) );
		$this->MapSetColor ('black', $this->allocateColor (0, 0, 0) );
		$this->MapSetColor ('green', $this->allocateColor (178, 237, 90) );
		$this->MapSetColor ('blue', $this->allocateColor (148, 208, 255) );
		$this->MapSetColor ('grey', $this->allocateColor (192, 192, 192) );
		$this->MapSetColor ('darkgrey', $this->allocateColor (124, 124, 124) );

	}

	/**
	* Sets the range of the map from overgiven degree-values
	*
	* container for API compatibility with PEAR::Image_GIS
	*
	* @access  public
	* @param   float   $x1
	* @param   float   $x2
	* @param   float   $y1
	* @param   float   $y2
	* @return  void
	*/

	function setRange ($x1, $x2, $y1, $y2) {

		$this->set_range ($x1, $x2, $y1, $y2);

	}

	/**
	* Sets the range of the map from overgiven GeoObjects
	*
	* @access  public
	* @param   array   &$geoObjects  Array of GeoObjects
	* @param   float   $border       degrees
	* @return  void
	* @see     setRange(),setRangeByGeoObject()
	*/

	function setRangeByGeoObjects (&$geoObjects, $border = 0.1) {

		foreach ($geoObjects as $geoObject) {
			if (!$this->longitudeMin || ($geoObject->longitude<$this->longitudeMin) ) {
				$this->longitudeMin = $geoObject->longitude;
			}
			if (!$this->longitudeMax || ($geoObject->longitude>$this->longitudeMax) ) {
				$this->longitudeMax = $geoObject->longitude;
			}
			if (!$this->latitudeMin || ($geoObject->latitude<$this->latitudeMin) ) {
				$this->latitudeMin = $geoObject->latitude;
			}
			if (!$this->latitudeMax || ($geoObject->latitude>$this->latitudeMax) ) {
				$this->latitudeMax = $geoObject->latitude;
			}
		}
		$this->setRange ($this->longitudeMin- $border, $this->longitudeMax+ $border, $this->latitudeMin- $border, $this->latitudeMax+ $border);

	}

	/**
	* Sets the range of the map from an overgiven GeoObject
	*
	* @access  public
	* @param   array   &$geoObject  GeoObject
	* @param   float   $border      degrees
	* @return  void
	* @see     setRange(),setRangeByGeoObjects()
	*/

	function setRangeByGeoObject (&$geoObject, $border = 0.1) {
		if (!$this->longitudeMin || ($geoObject->longitude<$this->longitudeMin) ) {
			$this->longitudeMin = $geoObject->longitude;
		}
		if (!$this->longitudeMax || ($geoObject->longitude>$this->longitudeMax) ) {
			$this->longitudeMax = $geoObject->longitude;
		}
		if (!$this->latitudeMin || ($geoObject->latitude<$this->latitudeMin) ) {
			$this->latitudeMin = $geoObject->latitude;
		}
		if (!$this->latitudeMax || ($geoObject->latitude>$this->latitudeMax) ) {
			$this->latitudeMax = $geoObject->latitude;
		}
		$this->setRange ($this->longitudeMin- $border, $this->longitudeMax+ $border, $this->latitudeMin- $border, $this->latitudeMax+ $border);

	}

	/**
	* Adds a GeoObject to the map
	*
	* @access  public
	* @param   array   &$geoObject  GeoObject
	* @param   string  $color
	* @param   int     $radius
	* @return  void
	* @see     addGeoObjects()
	*/

	function addGeoObject (&$geoObject, $color = 'black', $radius = false, $nofill = false) {
		if (!$radius) {
			$radius = $this->radius;
		}
		if ($radius) {
			if (!$nofill) {
				for ($i = 1; $i<= $radius; $i++) {
					ImageArc ($this->img, $this->scale ($geoObject->longitude, 'x'), $this->scale ($geoObject->latitude, 'y'), $i, $i, 0, 360, $this->color[$color]);
				}
			} else {
				ImageArc ($this->img, $this->scale ($geoObject->longitude, 'x'), $this->scale ($geoObject->latitude, 'y'), $radius, $radius, 0, 360, $this->color[$color]);
			}
		}
		$this->imageMap[] = array ('name' => $geoObject->getInfo (),
					'x' => $this->scale ($geoObject->longitude,
					'x'),
					'y' => $this->scale ($geoObject->latitude,
					'y'),
					'r' => $radius? $radius : $this->radius);

	}

	/**
	* Adds GeoObjects to the map
	*
	* @access  public
	* @param   array   &$geoObjects  Array of GeoObjects
	* @param   string  $color
	* @return  void
	* @see     addGeoObject()
	*/

	function addGeoObjects (&$geoObjects, $color = 'black') {

		foreach ($geoObjects as $geoObject) {
			$this->addGeoObject ($geoObject, $color);
		}

	}

	/**
	* Adds an e00-file to the image
	*
	* container for API compatibility with PEAR::Image_GIS
	*
	* @access  public
	* @param   string  $data  path to e00-file
	* @return  boolean
	* @see     map::draw()
	*/

	function addDataFile ($data, $color = 'black') {
		if (file_exists ($data) ) {
			$this->draw ($data, $this->color[$color]);
			return true;
		}
		return false;

	}

	/**
	* Saves the image
	*
	* container for API compatibility with PEAR::Image_GIS
	*
	* @access  public
	* @param   string  $file
	* @return  void
	* @see     map::dump()
	*/

	function saveImage ($file) {

		$this->dump ($file);

	}

	/**
	* Creates an image map (html)
	*
	* @access  public
	* @param   string  $name  name of the ImageMap
	* @return  string  html
	*/

	function getImageMap ($name = 'map') {

		$html = '<map name="' . $name . '">';
		foreach ($this->imageMap AS $koord) {
			$html .= '<area shape="circle" coords="' . round ($koord['x']) . ',' . round ($koord['y']) . ',' . $koord['r'] . '" href="#" alt="' . $koord['name'] . '">';
		}
		$html .= '</map>';
		return $html;

	}

	/**
	* Sets the colors for key
	*
	* @access  public
	* @param   string   $key
	* @param   array   $colors
	* @return  void
	*/

	function MapSetColor ($key, $colors) {

		$this->color[$key] = $colors;

	}

}

?>