<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

if (!defined ('_OPN_MAINFILE_INCLUDED') ) { die(); }

if (!defined ('_OPN_CLASS_MODULEFAMILY_INCLUDED') ) {
	define ('_OPN_CLASS_MODULEFAMILY_INCLUDED', 1);
	getLanguageDef (_OPN_ROOT_PATH . 'language/opn_management_class/language/');

	class opn_modulefamily {
		var $families;

		/**
		* Initialize of this class
		* fills with the var elements
		*
		* @access private
		* @return void
		**/

		function opn_modulefamily () {

			$this->families = '';

		}

		/**
		* Loads information to the $this->families
		* gets the familiy information from database
		*
		* @access privat
		* @param string $onlyformodule - if given, load only information for this family
		* @return void
		**/

		function load_families ($onlyformodule = '') {

			global $opnConfig, $opnTables;
			if ($onlyformodule != '') {
				$where = ' WHERE parentmodulename=' . $onlyformodule;
			} else {
				$where = '';
			}
			$sql = 'SELECT famid, parentmodulename, childmodulename, defaultdatastorage, childinuse, childtableprefix' . ' FROM ' . $opnTables['modulefamily_families'] . $where;
			$myfam = &$opnConfig['database']->Execute ($sql);
			if ( (!$myfam === false) && (!$myfam->EOF) ) {
				$this->families = array ();
				while (! $myfam->EOF) {
					$parent = $myfam->fields['parentmodulename'];
					$child = $myfam->fields['childmodulename'];
					$this->families[$parent][$child]['famid'] = $myfam->fields['famid'];
					$this->families[$parent][$child]['famid'] = $myfam->fields['defaultdatastorage'];
					$this->families[$parent][$child]['famid'] = $myfam->fields['childinuse'];
					$this->families[$parent][$child]['famid'] = $myfam->fields['childtableprefix'];
					$myfam->MoveNext ();
				}
			} else {
				$this->families = '';
			}

		}

		/**
		* adds family to the database
		*
		* @access privat
		* @param string $parentmodule - name of the parent module
		* @param string $childtableprefix - if individual data storage is desired, use this table prefix for them
		* @param array $childs - all the possible children with the desired default datastorage method (central or individual)
		* @return void
		**/

		function store_family ($parentmodule, $childtableprefix, $childs) {

			/* for prevention of double entries get current db */

			$onlyformodule = '';
			$this->load_families ($onlyformodule);
			if (!isset ($this->families[$parentmodule]) ) {
				if (!is_array ($childs) ) {
					$childs = array ($childs => 0);
				}
				foreach ($childs as $childname => $defaultdatastorage) {
					$this->store_familymember ($parentmodule, array ($childname => $defaultdatastorage),
											$childtableprefix);
				}
			} else {

				/* repair family */
			}
			$onlyformodule = '';
			$this->load_families ($onlyformodule);

		}

		/**
		* adds a familymember to the database
		*
		* @access privat
		* @param string $parentmodule - name of the parent module
		* @param array $child - child with the desired default datastorage method (central or individual)
		* @param string $childtableprefix - if individual data storage is desired, use this table prefix for them
		* @return void
		**/

		function store_familymember ($parentmodule, $child, $childtableprefix) {

			global $opnConfig, $opnTables;

			/* for prevention of double entries get current db */

			$onlyformodule = '';
			$this->load_families ($onlyformodule);
			if (!is_array ($child) ) {
				$child = array ($child => 1);
			}
			$childname = key ($child);
			$defaultdatastorage = current ($child);
			if (!isset ($this->families[$parentmodule][$childname]) ) {
				$famid = $opnConfig['opnSQL']->get_new_number ('modulefamily_families', 'famid');
				$sql = 'INSERT INTO ' . $opnTables['modulefamily_families'] . ' VALUES (' . $famid . ', ' . $opnConfig['opnSQL']->qstr ($parentmodule) . ', ' . $opnConfig['opnSQL']->qstr ($childname) . ', ' . $defaultdatastorage . ', 0' . ', ' . $opnConfig['opnSQL']->qstr ($childtableprefix) . ')';
				$opnConfig['database']->Execute ($sql);
				if ( ($defaultdatastorage == 1) && ($opnConfig['installedPlugins']->isplugininstalled ($childname) ) ) {
					if (!defined ('_OPN_CLASS_INSTALLER_INCLUDED') ) {
						include (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'class.installer.php');
					}
					$file = _OPN_ROOT_PATH . $childname . '/plugin/sql/index.php';
					if (file_exists ($file) ) {
						include_once ($file);
						$tables = array ();
						$myfuncSQLt = (substr ($childname, strpos ($childname, '/')+1) ) . '_repair_sql_table';
						$tables = $myfuncSQLt ();
						$renamedtables = array ();
						foreach ($tables['table'] as $tablename => $tablesettings) {
							$renamedtables['table'][$childtableprefix . '_' . $tablename] = $tablesettings;
						}
						$inst = new OPN_PluginInstaller;
						$inst->opnExecuteSQL ($renamedtables);
						unset ($inst);
						$tablenames = array_keys ($tables['table']);
						foreach ($tablenames as $tablename) {
							$ctabid = $opnConfig['opnSQL']->get_new_number ('modulefamily_childtables', 'ctabid');
							$sql = 'INSERT INTO ' . $opnTables['modulefamily_childtables'] . ' VALUES (' . $ctabid . ', ' . $opnConfig['opnSQL']->qstr ($childname) . ', ' . $opnConfig['opnSQL']->qstr ($tablename) . ', ' . $opnConfig['opnSQL']->qstr ($parentmodule) . ', ' . $opnConfig['opnSQL']->qstr ($childtableprefix . '_' . $tablename) . ')';
							$opnConfig['database']->Execute ($sql);
						}
					}
				}
			} else {

				/* repair familymember */
			}
			$onlyformodule = '';
			$this->load_families ($onlyformodule);

		}

		/**
		* removes family from the database
		*
		* @access privat
		* @param string $parentmodule - name of the parent module
		* @param array $childs - (optional) if set, only this member(s) are removed
		* @param bool $droptables - drop individual child tables
		* @return void
		**/

		function remove_family ($parentmodule, $childs = '', $droptables = false) {

			global $opnConfig, $opnTables;

			$t = $droptables;

			/* for prevention of double entries get current db */

			$onlyformodule = '';
			$this->load_families ($onlyformodule);
			if ( (isset ($this->families[$parentmodule]) ) || ( ($parentmodule == '') && ($childs != '') ) ) {
				$where = '';
				if (is_array ($childs) ) {
					$this->array_autoquote ($childs);
					$where = ' AND childmodulename IN (' . implode (', ', $childs) . ')';
				} elseif ($childs != '') {
					$where = ' AND childmodulename=' . $childs . ')';
				}
				$sql = 'DELETE FROM ' . $opnTables['modulefamily_families'] . ' WHERE parentmodulename=' . $opnConfig['opnSQL']->qstr ($parentmodule) . $where;
				$opnConfig['database']->Execute ($sql);
				$sql = 'DELETE FROM ' . $opnTables['modulefamily_childtables'] . ' WHERE parentmodulename=' . $opnConfig['opnSQL']->qstr ($parentmodule) . $where;
				$opnConfig['database']->Execute ($sql);
			}
			$onlyformodule = '';
			$this->load_families ($onlyformodule);

		}

		/**
		* autoquote all array items
		*
		* @access privat
		* @param array $quoteme - array items to quote
		* @return void
		**/

		function array_autoquote (&$quoteme) {

			global $opnConfig;
			if (is_array ($quoteme) ) {
				foreach ($quoteme as $key => $val) {
					$quoteme[$key] = $opnConfig['opnSQL']->qstr ($val);
				}
			}

		}

	}
}

?>